import logging
import requests
import werkzeug.urls

from datetime import datetime

try:
    from const import TIMEOUT, PLAUSIBLE_SAAS_SERVER, PLAUSIBLE_SAAS_APIKEY
except ImportError:
    from .const import TIMEOUT, PLAUSIBLE_SAAS_SERVER, PLAUSIBLE_SAAS_APIKEY


_logger = logging.getLogger(__name__)


class PlausibleAPI:
    def _request(self, method, url, headers=None, json=None, params=None):
        headers = headers or {}
        headers['authorization'] = 'Bearer %s' % PLAUSIBLE_SAAS_APIKEY
        url = PLAUSIBLE_SAAS_SERVER + url
        response = requests.request(method, url, headers=headers, json=json, params=params, timeout=TIMEOUT)
        _logger.info('Plausible: %s [%s] responsed with %s - %s', response.url, method, response.text, response.status_code)
        return response

    # WEBSITE
    def create_website(self, domain, tz=None):
        data = {
            'domain': domain,
        }
        if tz:
            data['timezone'] = tz
        response = self._request('POST', '/api/v1/sites', params=data)
        return response.json()

    def delete_website(self, domain):
        response = self._request('DELETE', '/api/v1/sites/' + (domain))
        return response.json()

    # SHARED LINKS
    def create_sharedlinks(self, domain, name=False):
        data = {
            'site_id': domain,
            'name': name or 'Shared Links %s' % datetime.now(),
        }
        response = self._request('PUT', '/api/v1/sites/shared-links', params=data)
        return response.json()

    # GOAL
    def create_goal(self, domain, goal_type, name):
        data = {
            'site_id': domain,
            'goal_type': goal_type,
        }
        if goal_type == 'page':
            data['page_path'] = name
        elif goal_type == 'event':
            data['event_name'] = name
        else:
            assert goal_type in ('event', 'page'), "PlausibleAPI: goal_type should be 'page' or 'event'"
        response = self._request('PUT', '/api/v1/sites/goals', params=data)
        return response.json()

    def delete_goal(self, domain, goal):
        data = {
            'site_id': domain,
        }
        response = self._request('DELETE', '/api/v1/sites/goals/' + werkzeug.urls.url_fix(goal), params=data)
        return response.json()

    def delete_goals(self, domain):
        response = self._request('DELETE', '/api/v1/sites/goals/website/' + domain)
        return response.json()

    def ping(self, domain):
        # Page view to disable the "Waiting for first pageview on ..."
        self._request('POST', '/api/event', json={
            'domain': domain,
            'name': "pageview",
            'url': "https://%s/Powered-by-Odoo" % domain,
            'screen_width': 1234
        })
        # Push custom event
        self._request('POST', '/api/event', json={
            'domain': domain,
            'name': "Alive",
            'url': "https://%s/ping" % domain
        })
